function [structure] = struct_cf_admm_ap(f, options)
% INPUT:
%   f: image can be n*r*ch, assume each pixel in [0,1]
%   options: list of options; currently:
%      b: parameter of the butterworth filter
%      lambda: the coefficient of the fitting term
%      alpha: coefficient in a(x,y) = alpha * D
%
% OUTPUT:
%   structure: structure of the image
%


% get options
options.null = 0; % trick to make sure that options is defined
method = getoptions(options, 'method', 'weighted_curv');
b = getoptions(options, 'b', 80);
lambda = getoptions(options, 'lambda', 70);
alpha = getoptions(options, 'alpha', 8);


[rows cols frames] = size(f);

H = fspecial('gaussian',[7,7],1);
g = imfilter(f,H);


switch method
    
    case 'curv'
        p = 1;
        u = f;
        K0 = curv_p(f,p); % clean curvature
        butter = lbutter(g,b,10); % low-pass Butterworth filter
        for c=1:size(g,3)
            K(:,:,c) = ifft2(ifftshift(fftshift(fft2(K0(:,:,c))).*butter));
        end % smoothed curvature
        
        Kp=K;
        
        for iter = 1:2
            % Setup parameters (for example)
            opts.rho_r   = 4;
            opts.rho_o   = 100;
            opts.beta    = [1 1 0];
            opts.print   = false;
            opts.alpha   = 0.7;
            opts.method  = 'l2';
            % Main routine
            opts.max_itr  =  20; % niter;
            out = deconvtv_p(g, H, lambda, Kp, p, 1, opts);
            u = out.f;
            if (iter == 1)
                %figure,imshow(u),title(['iter = ' num2str(iter)]);
            end
            
            g  = imfilter((u+f)/2,H);
            
            K_new = curv(u);
            D = abs(K_new-K)/max(abs(K_new(:)-K(:)));
            H3 = fspecial('gaussian',[7,7],3);
            D = imfilter(D,H3);
            
            Dsum = sum(D,3)/size(D,3);
            for c=1:size(g,3)
                D(:,:,c) = Dsum;
            end
            
            D = D./max(D(:));
            
            p = alpha*(1-0.5*D); % we update p(x,y)
            if (iter == 1)
                %figure,imshow(p),title(['iter = ' num2str(iter)]);
            end
            
            Kp = curv_p(f,p); % we have new p(x,y)
            for c=1:size(g,3)
                Kp(:,:,c) = ifft2(ifftshift(fftshift(fft2(Kp(:,:,c))).*butter));
            end % we smooth it
            
        end
        
        
    case 'weighted_curv'
        p = 1;
        a = 1;
        u = f;
        K0 = curv(f); % clean curvature
        butter = lbutter(g,b,10); % low-pass Butterworth filter
        for c=1:size(g,3)
            K(:,:,c) = ifft2(ifftshift(fftshift(fft2(K0(:,:,c))).*butter));
        end % smoothed curvature
        
        Kap=K;
        
        for iter = 1:2
            % Setup parameters (for example)
            opts.rho_r   = 4;
            opts.rho_o   = 100;
            opts.beta    = [1 1 0];
            opts.print   = false;
            opts.alpha   = 0.7;
            opts.method  = 'l2';
            % Main routine
            opts.max_itr  =  20; % niter;
            out = deconvtv_p(g, H, lambda, Kap, p, a, opts);
            u = out.f;
            if (iter == 1)
                %figure,imshow(u),title(['after iter ' num2str(iter)]);
            end
            
            g  = imfilter((u+f)/2,H);
            
            K_new = curv(u);
            D = abs(K_new-K)/max(abs(K_new(:)-K(:)));
            H3 = fspecial('gaussian',[7,7],3);
            D = imfilter(D,H3);
            
            Dsum = sum(D,3)/size(D,3);
            for c=1:size(g,3)
                D(:,:,c) = Dsum;
            end
            
            D = D./max(D(:));
            
            a = alpha*D; % updating a(x,y)
            p = (1-0.5*D); % updating p(x,y)
            
            if (iter == 1)
                %figure,imshow(D),title(['D(x,y) after iter ' num2str(iter)]);
            end
            
            Kap = curv_ap(f,a,p); % we have new weight a(x,y)
            for c=1:size(g,3)
                Kap(:,:,c) = ifft2(ifftshift(fftshift(fft2(Kap(:,:,c))).*butter));
            end % we smooth it
        end
        
        
    case 'grad'
        u = f;
        for iter = 1:2
            for c=1:size(g,3)
                D(:,:,c) = texture_weights((f+u)/2,3);
            end
            p = 0.5*(1+D).^2;
            
            % Setup parameters (for example)
            opts.rho_r   = 4;
            opts.rho_o   = 100;
            opts.beta    = [1 1 0];
            opts.print   = false;
            opts.alpha   = 0.7;
            opts.method  = 'l2';
            % Main routine
            opts.max_itr  =  20; % niter;
            out = deconvtv_p(g, H, lambda, 0, p, 1, opts);
            u = out.f;
            
        end
        
    otherwise
        error('unknown method \n');
end

structure = u;

end

